@@ -39,7 +39,7 @@ import numpy as np
 try:
     from scipy.stats.mstats import mquantiles
 except ImportError:
-    from pystan.external.mstats import mquantiles
+    from pystan.external.scipy.mstats import mquantiles
 
 import pystan.chains as chains
 from pystan.constants import (MAX_UINT, sampling_algo_t, optim_algo_t,
@@ -555,8 +555,10 @@ def _get_valid_stan_args(base_args=None):
     init = args.get('init', "random")
     if isinstance(init, string_types):
         args['init'] = init.encode('ascii')
-    elif isinstance(init, Sequence):
+    elif isinstance(init, dict):
         args['init'] = "user".encode('ascii')
+        # while the name is 'init_list', it is a dict; the name comes from rstan,
+        # where list elements can have names
         args['init_list'] = init
     else:
         args['init'] = "random".encode('ascii')
