@@ -42,8 +42,8 @@ add_binary("atan2");
 add_unary("atanh");
 for (size_t i = 0; i < int_vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) {
-    add("bernoulli_cdf",DOUBLE_T,int_vector_types[i],vector_types[j]);
     add("bernoulli_ccdf_log",DOUBLE_T,int_vector_types[i],vector_types[j]);
+    add("bernoulli_cdf",DOUBLE_T,int_vector_types[i],vector_types[j]);
     add("bernoulli_cdf_log",DOUBLE_T,int_vector_types[i],vector_types[j]);
     add("bernoulli_log",DOUBLE_T,int_vector_types[i],vector_types[j]);
     add("bernoulli_logit_log",DOUBLE_T,int_vector_types[i],vector_types[j]);
@@ -55,10 +55,10 @@ for (size_t i = 0; i < int_vector_types.size(); i++)
   for (size_t j = 0; j < int_vector_types.size(); j++)
     for (size_t k = 0; k < vector_types.size(); k++)
       for (size_t l = 0; l < vector_types.size(); l++) {
-        add("beta_binomial_cdf",DOUBLE_T, // result
+        add("beta_binomial_ccdf_log",DOUBLE_T, // result
             int_vector_types[i],int_vector_types[j],vector_types[k],
             vector_types[l]);
-        add("beta_binomial_ccdf_log",DOUBLE_T, // result
+        add("beta_binomial_cdf",DOUBLE_T, // result
             int_vector_types[i],int_vector_types[j],vector_types[k],
             vector_types[l]);
         add("beta_binomial_cdf_log",DOUBLE_T, // result
@@ -72,9 +72,6 @@ add("beta_binomial_rng",INT_T,INT_T,DOUBLE_T,DOUBLE_T);
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j)
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("beta_log",
-          DOUBLE_T, // result
-          vector_types[i], vector_types[j], vector_types[k]); // args
       add("beta_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
@@ -84,34 +81,49 @@ for (size_t i = 0; i < vector_types.size(); ++i)
       add("beta_cdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
+      add("beta_log",
+          DOUBLE_T, // result
+          vector_types[i], vector_types[j], vector_types[k]); // args
     }
 add_binary("beta_rng");
 add("binary_log_loss",DOUBLE_T,INT_T,DOUBLE_T);
-add_binary("binomial_coefficient_log");
-for (size_t i = 0; i < int_vector_types.size(); ++i) 
-  for (size_t j = 0; j < int_vector_types.size(); ++j)
+for (size_t i = 0; i < int_vector_types.size(); ++i) {
+  for (size_t j = 0; j < int_vector_types.size(); ++j) {
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("binomial_cdf",DOUBLE_T,
-          int_vector_types[i],int_vector_types[j],vector_types[k]);
       add("binomial_ccdf_log",DOUBLE_T,
           int_vector_types[i],int_vector_types[j],vector_types[k]);
+      add("binomial_cdf",DOUBLE_T,
+          int_vector_types[i],int_vector_types[j],vector_types[k]);
       add("binomial_cdf_log",DOUBLE_T,
           int_vector_types[i],int_vector_types[j],vector_types[k]);
+    }
+  }
+}
+add_binary("binomial_coefficient_log");
+for (size_t i = 0; i < int_vector_types.size(); ++i) {
+  for (size_t j = 0; j < int_vector_types.size(); ++j) {
+    for (size_t k = 0; k < vector_types.size(); ++k) {
       add("binomial_log",DOUBLE_T,
           int_vector_types[i],int_vector_types[j],vector_types[k]);
       add("binomial_logit_log",DOUBLE_T,
           int_vector_types[i],int_vector_types[j],vector_types[k]);
     }
+  }
+}
 add("binomial_rng",INT_T,INT_T,DOUBLE_T);
-add("categorical_log",DOUBLE_T,INT_T,VECTOR_T);
+add("block",MATRIX_T,MATRIX_T,INT_T,INT_T,INT_T,INT_T);
+for (size_t i = 0; i < int_vector_types.size(); ++i) {
+  add("categorical_log",DOUBLE_T, int_vector_types[i],VECTOR_T);
+  add("categorical_logit_log",DOUBLE_T, int_vector_types[i],VECTOR_T);
+}
 add("categorical_rng",INT_T,VECTOR_T);
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j)
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("cauchy_cdf",
+      add("cauchy_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("cauchy_ccdf_log",
+      add("cauchy_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("cauchy_cdf_log",
@@ -124,13 +136,12 @@ for (size_t i = 0; i < vector_types.size(); ++i)
 add_binary("cauchy_rng");
 add_unary("cbrt");
 add_unary("ceil");
-add("cholesky_decompose",MATRIX_T,MATRIX_T);
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j) {
-      add("chi_square_cdf",
+      add("chi_square_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j]); // args
-      add("chi_square_ccdf_log",
+      add("chi_square_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j]); // args
       add("chi_square_cdf_log",
@@ -141,7 +152,7 @@ for (size_t i = 0; i < vector_types.size(); ++i)
           vector_types[i], vector_types[j]); // args
   }
 add_unary("chi_square_rng");
-add("block",MATRIX_T,MATRIX_T,INT_T,INT_T,INT_T,INT_T);
+add("cholesky_decompose",MATRIX_T,MATRIX_T);
 add("col",VECTOR_T,MATRIX_T,INT_T);
 add("cols",INT_T,VECTOR_T);
 add("cols",INT_T,ROW_VECTOR_T);
@@ -154,34 +165,31 @@ add("columns_dot_self",ROW_VECTOR_T,ROW_VECTOR_T);
 add("columns_dot_self",ROW_VECTOR_T,MATRIX_T);
 add_unary("cos");
 add_unary("cosh");
+add("crossprod",MATRIX_T,MATRIX_T);
 add("cumulative_sum", expr_type(DOUBLE_T,1U), expr_type(DOUBLE_T,1U));
 add("cumulative_sum", VECTOR_T, VECTOR_T);
 add("cumulative_sum", ROW_VECTOR_T, ROW_VECTOR_T);
 add("determinant",DOUBLE_T,MATRIX_T);
-add("diagonal",VECTOR_T,MATRIX_T);
 add("diag_matrix",MATRIX_T,VECTOR_T);
+add("diagonal",VECTOR_T,MATRIX_T);
+// dims() is polymorphic by size
+for (size_t i = 0; i < 8; ++i) {
+  add("dims",expr_type(INT_T,1),expr_type(INT_T,i));
+  add("dims",expr_type(INT_T,1),expr_type(DOUBLE_T,i));
+  add("dims",expr_type(INT_T,1),expr_type(VECTOR_T,i));
+  add("dims",expr_type(INT_T,1),expr_type(ROW_VECTOR_T,i));
+  add("dims",expr_type(INT_T,1),expr_type(MATRIX_T,i));
+}
 add("dirichlet_log",DOUBLE_T,VECTOR_T,VECTOR_T);
 add("dirichlet_rng",VECTOR_T,VECTOR_T);
-add("divide",DOUBLE_T,DOUBLE_T,DOUBLE_T);
-add("divide",VECTOR_T,VECTOR_T,DOUBLE_T);
-add("divide",ROW_VECTOR_T,ROW_VECTOR_T,DOUBLE_T);
-add("divide",MATRIX_T,MATRIX_T,DOUBLE_T);
 add("dist",DOUBLE_T,VECTOR_T,VECTOR_T);
 add("dist",DOUBLE_T,ROW_VECTOR_T,ROW_VECTOR_T);
 add("dist",DOUBLE_T,VECTOR_T,ROW_VECTOR_T);
 add("dist",DOUBLE_T,ROW_VECTOR_T,VECTOR_T);
-add("squared_dist",DOUBLE_T,VECTOR_T,VECTOR_T);
-add("squared_dist",DOUBLE_T,ROW_VECTOR_T,ROW_VECTOR_T);
-add("squared_dist",DOUBLE_T,VECTOR_T,ROW_VECTOR_T);
-add("squared_dist",DOUBLE_T,ROW_VECTOR_T,VECTOR_T);
-add("squared_dist",DOUBLE_T,MATRIX_T,MATRIX_T);
-add("quad_form",DOUBLE_T,MATRIX_T,VECTOR_T);
-add("quad_form",MATRIX_T,MATRIX_T,MATRIX_T);
-add("quad_form_sym",DOUBLE_T,MATRIX_T,VECTOR_T);
-add("quad_form_sym",MATRIX_T,MATRIX_T,MATRIX_T);
-add("trace_quad_form",DOUBLE_T,MATRIX_T,VECTOR_T);
-add("trace_quad_form",DOUBLE_T,MATRIX_T,MATRIX_T);
-add("trace_gen_quad_form",DOUBLE_T,MATRIX_T,MATRIX_T,MATRIX_T);
+add("divide",DOUBLE_T,DOUBLE_T,DOUBLE_T);
+add("divide",VECTOR_T,VECTOR_T,DOUBLE_T);
+add("divide",ROW_VECTOR_T,ROW_VECTOR_T,DOUBLE_T);
+add("divide",MATRIX_T,MATRIX_T,DOUBLE_T);
 add("dot_product",DOUBLE_T,VECTOR_T,VECTOR_T);
 add("dot_product",DOUBLE_T,ROW_VECTOR_T,ROW_VECTOR_T);
 add("dot_product",DOUBLE_T,VECTOR_T,ROW_VECTOR_T);
@@ -192,10 +200,10 @@ add("dot_self",DOUBLE_T,ROW_VECTOR_T);
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j)
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("double_exponential_cdf",
+      add("double_exponential_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("double_exponential_ccdf_log",
+      add("double_exponential_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("double_exponential_cdf_log",
@@ -215,13 +223,6 @@ add("elt_divide",MATRIX_T,MATRIX_T,MATRIX_T);
 add("elt_multiply",VECTOR_T,VECTOR_T,VECTOR_T);
 add("elt_multiply",ROW_VECTOR_T,ROW_VECTOR_T,ROW_VECTOR_T);
 add("elt_multiply",MATRIX_T,MATRIX_T,MATRIX_T);
-add("diag_pre_multiply",MATRIX_T,MATRIX_T,MATRIX_T);
-add("diag_pre_multiply",MATRIX_T,VECTOR_T,MATRIX_T);
-add("diag_pre_multiply",MATRIX_T,ROW_VECTOR_T,MATRIX_T);
-add("diag_post_multiply",MATRIX_T,MATRIX_T,MATRIX_T);
-add("diag_post_multiply",MATRIX_T,MATRIX_T,VECTOR_T);
-add("diag_post_multiply",MATRIX_T,MATRIX_T,ROW_VECTOR_T);
-add_nullary("epsilon");
 add_unary("erf");
 add_unary("erfc");
 add_unary("exp");
@@ -229,32 +230,15 @@ add("exp",VECTOR_T,VECTOR_T);
 add("exp",ROW_VECTOR_T,ROW_VECTOR_T);
 add("exp",MATRIX_T,MATRIX_T);
 add_unary("exp2");
-add_unary("expm1");
-for (size_t i = 0; i < vector_types.size(); ++i)
-  for (size_t j = 0; j < vector_types.size(); ++j) {
-      add("exponential_cdf",
-          DOUBLE_T, // result
-          vector_types[i], vector_types[j]); // args
-      add("exponential_ccdf_log",
-          DOUBLE_T, // result
-          vector_types[i], vector_types[j]); // args
-      add("exponential_cdf_log",
-          DOUBLE_T, // result
-          vector_types[i], vector_types[j]); // args
-      add("exponential_log",
-          DOUBLE_T, // result
-          vector_types[i], vector_types[j]); // args
-  }
-add_unary("exponential_rng");
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k)
       for (size_t l = 0; l < vector_types.size(); ++l) {
-        add("exp_mod_normal_cdf",
+        add("exp_mod_normal_ccdf_log",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k], 
             vector_types[l]); // args
-        add("exp_mod_normal_ccdf_log",
+        add("exp_mod_normal_cdf",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k], 
             vector_types[l]); // args
@@ -268,6 +252,23 @@ for (size_t i = 0; i < vector_types.size(); ++i)
             vector_types[l]); // args
       }
 add_ternary("exp_mod_normal_rng");
+add_unary("expm1");
+for (size_t i = 0; i < vector_types.size(); ++i)
+  for (size_t j = 0; j < vector_types.size(); ++j) {
+      add("exponential_ccdf_log",
+          DOUBLE_T, // result
+          vector_types[i], vector_types[j]); // args
+      add("exponential_cdf",
+          DOUBLE_T, // result
+          vector_types[i], vector_types[j]); // args
+      add("exponential_cdf_log",
+          DOUBLE_T, // result
+          vector_types[i], vector_types[j]); // args
+      add("exponential_log",
+          DOUBLE_T, // result
+          vector_types[i], vector_types[j]); // args
+  }
+add_unary("exponential_rng");
 add_unary("fabs");
 add_binary("falling_factorial");
 add_binary("fdim");
@@ -279,10 +280,10 @@ add_binary("fmod");
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("gamma_cdf",
+      add("gamma_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("gamma_ccdf_log",
+      add("gamma_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("gamma_cdf_log",
@@ -295,13 +296,15 @@ for (size_t i = 0; i < vector_types.size(); ++i)
 add_binary("gamma_p");
 add_binary("gamma_q");
 add_binary("gamma_rng");
+add("gaussian_dlm_obs_log",DOUBLE_T,MATRIX_T,MATRIX_T,MATRIX_T,MATRIX_T,MATRIX_T,VECTOR_T,MATRIX_T);
+add("gaussian_dlm_obs_log",DOUBLE_T,MATRIX_T,MATRIX_T,MATRIX_T,VECTOR_T,MATRIX_T,VECTOR_T,MATRIX_T);
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("gumbel_cdf",
+      add("gumbel_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("gumbel_ccdf_log",
+      add("gumbel_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("gumbel_cdf_log",
@@ -325,12 +328,13 @@ add_binary("hypot");
 add("if_else",DOUBLE_T,INT_T,DOUBLE_T,DOUBLE_T);
 add("int_step",INT_T,DOUBLE_T);
 add("int_step",INT_T,INT_T);
+add_unary("inv");
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j) {
-      add("inv_chi_square_cdf",
+      add("inv_chi_square_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j]); // args
-      add("inv_chi_square_ccdf_log",
+      add("inv_chi_square_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j]); // args
       add("inv_chi_square_cdf_log",
@@ -345,10 +349,10 @@ add_unary("inv_cloglog");
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("inv_gamma_cdf",
+      add("inv_gamma_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("inv_gamma_ccdf_log",
+      add("inv_gamma_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("inv_gamma_cdf_log",
@@ -360,11 +364,10 @@ for (size_t i = 0; i < vector_types.size(); ++i)
    }
 add_binary("inv_gamma_rng");
 add_unary("inv_logit");
-add_unary("inv_square");
 add_unary("inv_sqrt");
+add_unary("inv_square");
 add("inv_wishart_log",DOUBLE_T, MATRIX_T,DOUBLE_T,MATRIX_T);
 add("inv_wishart_rng",MATRIX_T,DOUBLE_T,MATRIX_T);
-add_unary("inv");
 add("inverse",MATRIX_T,MATRIX_T);
 add("inverse_spd",MATRIX_T,MATRIX_T);
 add_binary("lbeta");
@@ -375,6 +378,30 @@ add("lkj_corr_log",DOUBLE_T, MATRIX_T,DOUBLE_T);
 add("lkj_corr_rng",MATRIX_T,INT_T,DOUBLE_T);
 add("lkj_cov_log",DOUBLE_T, MATRIX_T,VECTOR_T,VECTOR_T,DOUBLE_T);
 add("lmgamma",DOUBLE_T,INT_T,DOUBLE_T);
+add_unary("log");
+add("log",VECTOR_T,VECTOR_T);
+add("log",ROW_VECTOR_T,ROW_VECTOR_T);
+add("log",MATRIX_T,MATRIX_T);
+add_nullary("log10");
+add_unary("log10");
+add_unary("log1m");
+add_unary("log1m_exp");
+add_unary("log1m_inv_logit");
+add_unary("log1p");
+add_unary("log1p_exp");
+add_nullary("log2");
+add_unary("log2");
+add("log_determinant",DOUBLE_T,MATRIX_T);
+add_binary("log_diff_exp");
+add_binary("log_falling_factorial");
+add_binary("log_rising_factorial");
+add_unary("log_inv_logit");
+add("log_softmax",VECTOR_T,VECTOR_T);
+add("log_sum_exp",DOUBLE_T, expr_type(DOUBLE_T,1U));
+add("log_sum_exp",DOUBLE_T, VECTOR_T);
+add("log_sum_exp",DOUBLE_T, ROW_VECTOR_T);
+add("log_sum_exp",DOUBLE_T, MATRIX_T);
+add_binary("log_sum_exp");
 for (size_t i = 0; i < primitive_types.size(); ++i) {
   add("logical_negation",INT_T,primitive_types[i]);
   for (size_t j = 0; j < primitive_types.size(); ++j) {
@@ -388,33 +415,13 @@ for (size_t i = 0; i < primitive_types.size(); ++i) {
     add("logical_gte",INT_T,primitive_types[i], primitive_types[j]);
   }
 }
-add_unary("log");
-add("log",VECTOR_T,VECTOR_T);
-add("log",ROW_VECTOR_T,ROW_VECTOR_T);
-add("log",MATRIX_T,MATRIX_T);
-add("log_determinant",DOUBLE_T,MATRIX_T);
-add_binary("log_falling_factorial");
-add_binary("log_rising_factorial");
-add("log_sum_exp",DOUBLE_T, expr_type(DOUBLE_T,1U));
-add_binary("log_sum_exp");
-add_nullary("log10");
-add_unary("log10");
-add_unary("log1m");
-add_unary("log1m_exp");
-add_unary("log1p");
-add_unary("log1p_exp");
-add_binary("log_diff_exp");
-add_unary("log_inv_logit");
-add_unary("log1m_inv_logit");
-add_nullary("log2");
-add_unary("log2");
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("logistic_cdf",
+      add("logistic_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("logistic_ccdf_log",
+      add("logistic_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("logistic_cdf_log",
@@ -429,10 +436,10 @@ add_unary("logit");
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("lognormal_cdf",
+      add("lognormal_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("lognormal_ccdf_log",
+      add("lognormal_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("lognormal_cdf_log",
@@ -443,6 +450,7 @@ for (size_t i = 0; i < vector_types.size(); ++i)
           vector_types[i], vector_types[j], vector_types[k]); // args
     }
 add_binary("lognormal_rng");
+add_nullary("machine_precision");
 add("max",INT_T,expr_type(INT_T,1));
 add("max",DOUBLE_T,expr_type(DOUBLE_T,1));
 add("max",DOUBLE_T,VECTOR_T);
@@ -451,10 +459,10 @@ add("max",DOUBLE_T,MATRIX_T);
 add("max",INT_T,INT_T,INT_T);
 add("mdivide_left",VECTOR_T,MATRIX_T,VECTOR_T);
 add("mdivide_left",MATRIX_T,MATRIX_T,MATRIX_T);
-add("mdivide_right",ROW_VECTOR_T,ROW_VECTOR_T,MATRIX_T);
-add("mdivide_right",MATRIX_T,MATRIX_T,MATRIX_T);
 add("mdivide_left_tri_low",MATRIX_T,MATRIX_T,MATRIX_T);
 add("mdivide_left_tri_low",VECTOR_T,MATRIX_T,VECTOR_T);
+add("mdivide_right",ROW_VECTOR_T,ROW_VECTOR_T,MATRIX_T);
+add("mdivide_right",MATRIX_T,MATRIX_T,MATRIX_T);
 add("mdivide_right_tri_low",ROW_VECTOR_T,ROW_VECTOR_T,MATRIX_T);
 add("mdivide_right_tri_low",MATRIX_T,MATRIX_T,MATRIX_T);
 add("mean",DOUBLE_T,expr_type(DOUBLE_T,1));
@@ -471,13 +479,11 @@ add("minus",DOUBLE_T,DOUBLE_T);
 add("minus",VECTOR_T,VECTOR_T);
 add("minus",ROW_VECTOR_T,ROW_VECTOR_T);
 add("minus",MATRIX_T,MATRIX_T);
-add("matrix_normal_prec_log",DOUBLE_T,MATRIX_T,MATRIX_T,MATRIX_T,MATRIX_T);
 add("modified_bessel_first_kind",DOUBLE_T,INT_T,DOUBLE_T);
 add("modified_bessel_second_kind",DOUBLE_T,INT_T,DOUBLE_T);
-add("multi_gp_log",DOUBLE_T,MATRIX_T,MATRIX_T,VECTOR_T);
 add("multi_normal_cholesky_log",DOUBLE_T, VECTOR_T,VECTOR_T,MATRIX_T);
-add("multi_normal_prec_log",DOUBLE_T, VECTOR_T,VECTOR_T,MATRIX_T);
 add("multi_normal_log",DOUBLE_T, VECTOR_T,VECTOR_T,MATRIX_T);
+add("multi_normal_prec_log",DOUBLE_T, VECTOR_T,VECTOR_T,MATRIX_T);
 add("multi_normal_rng",VECTOR_T,VECTOR_T,MATRIX_T);
 add("multi_student_t_log",DOUBLE_T, VECTOR_T,DOUBLE_T,VECTOR_T,MATRIX_T);
 add("multi_student_t_rng",VECTOR_T, DOUBLE_T,VECTOR_T,MATRIX_T);
@@ -495,32 +501,29 @@ add("multiply",MATRIX_T,MATRIX_T,MATRIX_T);
 add("multiply",VECTOR_T,DOUBLE_T,VECTOR_T);
 add("multiply",ROW_VECTOR_T,DOUBLE_T,ROW_VECTOR_T);
 add("multiply",MATRIX_T,DOUBLE_T,MATRIX_T);
-add("multiply_lower_tri_self_transpose",MATRIX_T,MATRIX_T);
-add("tcrossprod",MATRIX_T,MATRIX_T);
-add("crossprod",MATRIX_T,MATRIX_T);
 add_binary("multiply_log");
+add("multiply_lower_tri_self_transpose",MATRIX_T,MATRIX_T);
 for (size_t i = 0; i < int_vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("neg_binomial_cdf",DOUBLE_T,
-          int_vector_types[i],vector_types[j],vector_types[k]);
       add("neg_binomial_ccdf_log",DOUBLE_T,
           int_vector_types[i],vector_types[j],vector_types[k]);
+      add("neg_binomial_cdf",DOUBLE_T,
+          int_vector_types[i],vector_types[j],vector_types[k]);
       add("neg_binomial_cdf_log",DOUBLE_T,
           int_vector_types[i],vector_types[j],vector_types[k]);
       add("neg_binomial_log",DOUBLE_T,
           int_vector_types[i],vector_types[j],vector_types[k]);
     }
 add("neg_binomial_rng",INT_T,DOUBLE_T,DOUBLE_T);
-add_nullary("negative_epsilon");
 add_nullary("negative_infinity");
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("normal_cdf",
+      add("normal_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("normal_ccdf_log",
+      add("normal_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("normal_cdf_log",
@@ -538,10 +541,10 @@ add_binary("owens_t");
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("pareto_cdf",
+      add("pareto_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("pareto_ccdf_log",
+      add("pareto_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("pareto_cdf_log",
@@ -557,8 +560,8 @@ add_unary("Phi_approx");
 add_nullary("pi");
 for (size_t i = 0; i < int_vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j) {
-    add("poisson_cdf",DOUBLE_T, int_vector_types[i],vector_types[j]);
     add("poisson_ccdf_log",DOUBLE_T, int_vector_types[i],vector_types[j]);
+    add("poisson_cdf",DOUBLE_T, int_vector_types[i],vector_types[j]);
     add("poisson_cdf_log",DOUBLE_T, int_vector_types[i],vector_types[j]);
     add("poisson_log",DOUBLE_T, int_vector_types[i],vector_types[j]);
     add("poisson_log_log",DOUBLE_T, int_vector_types[i],vector_types[j]);
@@ -571,22 +574,24 @@ add("prod",DOUBLE_T,expr_type(DOUBLE_T,1));
 add("prod",DOUBLE_T,VECTOR_T);
 add("prod",DOUBLE_T,ROW_VECTOR_T);
 add("prod",DOUBLE_T,MATRIX_T);
+add("quad_form",DOUBLE_T,MATRIX_T,VECTOR_T);
+add("quad_form",MATRIX_T,MATRIX_T,MATRIX_T);
 add("rank",INT_T,expr_type(INT_T,1),INT_T);
 add("rank",INT_T,expr_type(DOUBLE_T,1),INT_T);
 add("rank",INT_T,VECTOR_T,INT_T);
 add("rank",INT_T,ROW_VECTOR_T,INT_T);
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j) {
-    add("rayleigh_log",
+    add("rayleigh_ccdf_log",
         DOUBLE_T, // result
         vector_types[i], vector_types[j]); // args
     add("rayleigh_cdf",
         DOUBLE_T, // result
         vector_types[i], vector_types[j]); // args
-    add("rayleigh_ccdf_log",
+    add("rayleigh_cdf_log",
         DOUBLE_T, // result
         vector_types[i], vector_types[j]); // args
-    add("rayleigh_cdf_log",
+    add("rayleigh_log",
         DOUBLE_T, // result
         vector_types[i], vector_types[j]); // args
   }
@@ -621,10 +626,10 @@ add("rows_dot_self",VECTOR_T,MATRIX_T);
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-      add("scaled_inv_chi_square_cdf",
+      add("scaled_inv_chi_square_ccdf_log",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
-      add("scaled_inv_chi_square_ccdf_log",
+      add("scaled_inv_chi_square_cdf",
           DOUBLE_T, // result
           vector_types[i], vector_types[j], vector_types[k]); // args
       add("scaled_inv_chi_square_cdf_log",
@@ -649,14 +654,22 @@ for (size_t i = 0; i < base_types.size(); ++i) {
 add_unary("sin");
 add("singular_values",VECTOR_T,MATRIX_T);
 add_unary("sinh");
+// size() is polymorphic over arrays, so start i at 1
+for (size_t i = 1; i < 8; ++i) {
+  add("size",INT_T,expr_type(INT_T,i));
+  add("size",INT_T,expr_type(DOUBLE_T,i));
+  add("size",INT_T,expr_type(VECTOR_T,i));
+  add("size",INT_T,expr_type(ROW_VECTOR_T,i));
+  add("size",INT_T,expr_type(MATRIX_T,i));
+}
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) 
       for (size_t l = 0; l < vector_types.size(); ++l) {
-        add("skew_normal_cdf",
+        add("skew_normal_ccdf_log",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k], vector_types[l]); // args
-        add("skew_normal_ccdf_log",
+        add("skew_normal_cdf",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k], vector_types[l]); // args
         add("skew_normal_cdf_log",
@@ -684,10 +697,10 @@ for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) 
       for (size_t l = 0; l < vector_types.size(); ++l) {
-        add("student_t_cdf",
+        add("student_t_ccdf_log",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k], vector_types[l]); // args
-        add("student_t_ccdf_log",
+        add("student_t_cdf",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k], vector_types[l]); // args
         add("student_t_cdf_log",
@@ -714,7 +727,6 @@ add("sum",DOUBLE_T,expr_type(DOUBLE_T,1));
 add("sum",DOUBLE_T,VECTOR_T);
 add("sum",DOUBLE_T,ROW_VECTOR_T);
 add("sum",DOUBLE_T,MATRIX_T);
-// see head() and segment()
 add("tail",ROW_VECTOR_T, ROW_VECTOR_T,INT_T);
 add("tail",VECTOR_T, VECTOR_T,INT_T);
 for (size_t i = 0; i < base_types.size(); ++i) {
@@ -724,10 +736,14 @@ for (size_t i = 0; i < base_types.size(); ++i) {
  }
 add_unary("tan");
 add_unary("tanh");
+add("tcrossprod",MATRIX_T,MATRIX_T);
 add_unary("tgamma");
 add("to_vector", VECTOR_T, ROW_VECTOR_T);
 add("to_vector", VECTOR_T, MATRIX_T);
 add("trace",DOUBLE_T,MATRIX_T);
+add("trace_gen_quad_form",DOUBLE_T,MATRIX_T,MATRIX_T,MATRIX_T);
+add("trace_quad_form",DOUBLE_T,MATRIX_T,VECTOR_T);
+add("trace_quad_form",DOUBLE_T,MATRIX_T,MATRIX_T);
 add("transpose",ROW_VECTOR_T,VECTOR_T);
 add("transpose",VECTOR_T,ROW_VECTOR_T);
 add("transpose",MATRIX_T,MATRIX_T);
@@ -735,10 +751,10 @@ add_unary("trunc");
 for (size_t i = 0; i < vector_types.size(); ++i) 
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-        add("uniform_cdf",
+        add("uniform_ccdf_log",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k]); // args
-        add("uniform_ccdf_log",
+        add("uniform_cdf",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k]); // args
         add("uniform_cdf_log",
@@ -753,13 +769,19 @@ add("variance",DOUBLE_T,expr_type(DOUBLE_T,1));
 add("variance",DOUBLE_T,VECTOR_T);
 add("variance",DOUBLE_T,ROW_VECTOR_T);
 add("variance",DOUBLE_T,MATRIX_T);
+for (size_t i = 0; i < vector_types.size(); ++i)
+  for (size_t j = 0; j < vector_types.size(); ++j)
+    for (size_t k = 0; k < vector_types.size(); ++k)
+      add("von_mises_log",
+          DOUBLE_T, // result
+          vector_types[i], vector_types[j], vector_types[k]); // args
 for (size_t i = 0; i < vector_types.size(); ++i)
   for (size_t j = 0; j < vector_types.size(); ++j) 
     for (size_t k = 0; k < vector_types.size(); ++k) {
-        add("weibull_cdf",
+        add("weibull_ccdf_log",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k]); // args
-        add("weibull_ccdf_log",
+        add("weibull_cdf",
             DOUBLE_T, // result
             vector_types[i], vector_types[j], vector_types[k]); // args
         add("weibull_cdf_log",
@@ -772,23 +794,3 @@ for (size_t i = 0; i < vector_types.size(); ++i)
 add_binary("weibull_rng");
 add("wishart_log",DOUBLE_T, MATRIX_T,DOUBLE_T,MATRIX_T);
 add("wishart_rng",MATRIX_T, DOUBLE_T,MATRIX_T);
-
-// dims() is polymorphic by size
-for (size_t i = 0; i < 8; ++i) {
-  add("dims",expr_type(INT_T,1),expr_type(INT_T,i));
-  add("dims",expr_type(INT_T,1),expr_type(DOUBLE_T,i));
-  add("dims",expr_type(INT_T,1),expr_type(VECTOR_T,i));
-  add("dims",expr_type(INT_T,1),expr_type(ROW_VECTOR_T,i));
-  add("dims",expr_type(INT_T,1),expr_type(MATRIX_T,i));
-}
-// size() is polymorphic over arrays, so start i at 1
-for (size_t i = 1; i < 8; ++i) {
-  add("size",INT_T,expr_type(INT_T,i));
-  add("size",INT_T,expr_type(DOUBLE_T,i));
-  add("size",INT_T,expr_type(VECTOR_T,i));
-  add("size",INT_T,expr_type(ROW_VECTOR_T,i));
-  add("size",INT_T,expr_type(MATRIX_T,i));
-}
-//------------------------------------------------------------
-
-add("trunc_normal_log",DOUBLE_T, DOUBLE_T,DOUBLE_T,DOUBLE_T,DOUBLE_T,DOUBLE_T);
