@@ -423,6 +423,19 @@ namespace stan {
                RNG& rng) {
       using boost::variate_generator;
       using boost::random::cauchy_distribution;
+
+      static const char* function = "stan::prob::cauchy_rng(%1%)";
+
+      using stan::math::check_positive;
+      using stan::math::check_finite;
+      
+      if (!check_finite(function, mu, "Location parameter"))
+        return 0;
+      if (!check_positive(function, sigma, "Scale parameter"))
+        return 0;
+      if (!check_finite(function, sigma, "Scale parameter"))
+        return 0;
+
       variate_generator<RNG&, cauchy_distribution<> >
         cauchy_rng(rng, cauchy_distribution<>(mu, sigma));
       return cauchy_rng();
