@@ -1,7 +1,7 @@
 #include <gtest/gtest.h>
 #include "stan/prob/distributions/multivariate/continuous/multi_normal.hpp"
 #include <boost/random/mersenne_twister.hpp>
-#include<boost/math/distributions.hpp>
+#include <boost/math/distributions.hpp>
 
 using Eigen::Dynamic;
 using Eigen::Matrix;
@@ -16,9 +16,8 @@ TEST(ProbDistributionsMultiNormal,MultiNormal) {
     -3.0,  4.0, 0.0,
     0.0, 0.0, 5.0;
   EXPECT_FLOAT_EQ(-11.73908, stan::prob::multi_normal_log(y,mu,Sigma));
-  Matrix<double,Dynamic,Dynamic> L = Sigma.llt().matrixL();
-  EXPECT_FLOAT_EQ(-11.73908, stan::prob::multi_normal_cholesky_log(y,mu,L));
 }
+
 TEST(ProbDistributionsMultiNormal,Sigma) {
   Matrix<double,Dynamic,1> y(2,1);
   y << 2.0, -2.0;
@@ -60,8 +59,6 @@ TEST(ProbDistributionsMultiNormal,MultiNormalOneRow) {
     -3.0,  4.0, 0.0,
     0.0, 0.0, 5.0;
   EXPECT_FLOAT_EQ(-11.73908, stan::prob::multi_normal_log(y,mu,Sigma));
-  Matrix<double,Dynamic,Dynamic> L = Sigma.llt().matrixL();
-  EXPECT_FLOAT_EQ(-11.73908, stan::prob::multi_normal_cholesky_log(y,mu,L));
 }
 
 TEST(ProbDistributionsMultiNormal,MultiNormalMultiRow) {
@@ -75,8 +72,6 @@ TEST(ProbDistributionsMultiNormal,MultiNormalMultiRow) {
     -3.0,  4.0, 0.0,
     0.0, 0.0, 5.0;
   EXPECT_FLOAT_EQ(-54.2152, stan::prob::multi_normal_log(y,mu,Sigma));
-  Matrix<double,Dynamic,Dynamic> L = Sigma.llt().matrixL();
-  EXPECT_FLOAT_EQ(-54.2152, stan::prob::multi_normal_cholesky_log(y,mu,L));
 }
 TEST(ProbDistributionsMultiNormal,SigmaMultiRow) {
   Matrix<double,Dynamic,Dynamic> y(1,2);
