@@ -3,9 +3,14 @@
 #include <boost/random/mersenne_twister.hpp>
 #include<boost/math/distributions.hpp>
 
-TEST(ProbDistributionsInvChiSquare, random) {
+TEST(ProbDistributionsInvChiSquare, error_check) {
   boost::random::mt19937 rng;
   EXPECT_NO_THROW(stan::prob::inv_chi_square_rng(4.0,rng));
+
+  EXPECT_THROW(stan::prob::inv_chi_square_rng(-4.0,rng),std::domain_error);
+  EXPECT_THROW(stan::prob::inv_chi_square_rng(stan::math::positive_infinity(),
+                                              rng),
+               std::domain_error);
 }
 
 TEST(ProbDistributionsInvChiSquare, chiSquareGoodnessFitTest) {
